package chat

import (
	"context"
	"errors"
	"fmt"

	"wechat-api/ent"
	"wechat-api/ent/predicate"
	"wechat-api/ent/wx"
	"wechat-api/internal/service/addfriend"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/typekit"

	"github.com/go-sql-driver/mysql"
	"github.com/suyuan32/simple-admin-common/enum/errorcode"
	"github.com/suyuan32/simple-admin-common/msg/errormsg"
	"github.com/zeromicro/go-zero/core/errorx"
	"github.com/zeromicro/go-zero/core/logx"
)

type AddFriendByPhoneLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewAddFriendByPhoneLogic(ctx context.Context, svcCtx *svc.ServiceContext) *AddFriendByPhoneLogic {
	return &AddFriendByPhoneLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx}
}

func GetListRandOne(strList []string) string {
	one := ""
	if len(strList) > 0 {
		typekit.ShuffleList(strList)
		one = strList[0]
	}
	return one
}

func (l *AddFriendByPhoneLogic) getWxInfoList(preds ...predicate.Wx) ([]*ent.Wx, error) {
	preds = append(preds, wx.StatusEQ(1))
	return l.svcCtx.DB.Wx.Query().Where(preds...).All(l.ctx)
}

func (l *AddFriendByPhoneLogic) getWxIdListByOid(Oid uint64, Type uint64) ([]string, error) {
	preds := []predicate.Wx{}
	preds = append(preds, wx.OrganizationIDEQ(Oid))
	if Type == 1 || Type == 3 {
		preds = append(preds, wx.CtypeEQ(Type))
	}
	wxIdList := []string{}
	wxInfoList, err := l.getWxInfoList(preds...)
	if err == nil {
		for _, wxInfo := range wxInfoList {
			wxIdList = append(wxIdList, wxInfo.Wxid)
		}
	}
	return wxIdList, err
}

type AddFriendByPhoneInfo struct {
	Type        int    `json:"type"`
	WeChatId    string `json:"wechat_id"`
	Oid         int64  `json:"oid"`
	Phone       string `json:"phone"`
	Message     string `json:"message"`
	CallbackURL string `json:"callback_url"`
}

func (l *AddFriendByPhoneLogic) rebuildAddFriendByPhoneInfos(req *types.AddFriendByPhoneReq,
	Oid uint64) ([]AddFriendByPhoneInfo, error) {

	var err error
	_, req.WeChatIds, req.Phones, err = l.alignWxidsAndPhones(Oid,
		uint64(req.Type), req.WeChatIds, req.Phones)
	if err != nil {
		return nil, err
	}
	infos := []AddFriendByPhoneInfo{}
	for idx, phone := range req.Phones { //以phones为主，逐一绑定wxid,多余的wxid忽略
		info := AddFriendByPhoneInfo{Type: req.Type, WeChatId: req.WeChatIds[idx],
			Phone: phone, Message: req.Message, CallbackURL: req.CallbackURL, Oid: int64(Oid)}
		infos = append(infos, info)
	}
	return infos, nil
}

/*
1.手机号是必传的，至少有1个
2.微信号可以不传为空，这种情况拿当前登录者同租户OID的微信ID随机绑定传入的每个手机号
3.微信号数量等于或者大于手机号数量，则按输入前后顺序逐一绑定，多出来的微信号忽略
4.当微信号数量小于手机号数量，多出来的手机号则从输入的微信号中随机找1个绑定
*/

func (l *AddFriendByPhoneLogic) alignWxidsAndPhones(Oid uint64, CType uint64,
	weChatIds []string, phones []string) (bool, []string, []string, error) {

	lenWechatIds := len(weChatIds)
	lenPhones := len(phones)

	if lenPhones == 0 {
		return false, nil, nil, errors.New("phones empty")
	}

	if lenWechatIds == 0 { //输入wxids为空,从当前库中同Oid的Wxid里随机选1个
		allWxIds, err := l.getWxIdListByOid(Oid, CType)
		if err != nil {
			return false, weChatIds, phones, err
		}
		if len(allWxIds) == 0 {
			return false, weChatIds, phones, fmt.Errorf("wxids empty(%d)", Oid)
		}
		for range lenPhones {
			weChatIds = append(weChatIds, GetListRandOne(allWxIds))
		}
	} else if lenWechatIds < lenPhones { //当wxids数量小于phones数量,从已输入wxids随机选1个对齐phones
		for range lenPhones - lenWechatIds {
			weChatIds = append(weChatIds, GetListRandOne(weChatIds))
		}
	}
	//剩下的case就是wxids数量等于或大于phones
	return true, weChatIds, phones, nil
}

/*
func (l *AddFriendByPhoneLogic) alignWxidsAndPhones(Oid uint64, CType uint64,
	weChatIds []string, phones []string) (bool, []string, []string, error) {

	lenWechatIds := len(weChatIds)
	lenPhones := len(phones)
	//case1: 输入的wxids和phones大小一样，不需要对齐
	if lenWechatIds == lenPhones {
		return true, weChatIds, phones, nil
	}
	//case2: 输入的wxids比phones小或者空
	if lenWechatIds < lenPhones {
		allWxIds, err := l.getWxIdListByOid(Oid, CType)
		if err != nil {
			return false, weChatIds, phones, err
		}
		if len(allWxIds) == 0 {
			if len(weChatIds) == 0 {
				err = errors.New("db wxIds empty")
			}
			return false, weChatIds, phones, err
		}
		for i := 0; i < lenPhones-lenWechatIds; i++ {
			weChatIds = append(weChatIds, GetRandWxid(allWxIds))
		}
	} else { //case 3: 输入的wxids比phones大

		//	for i := 0; i < lenWechatIds-lenPhones; i++ {
		//		phones = append(phones, GetRandWxid(phones))
		//	}

	}
	return true, weChatIds, phones, nil
}
*/

func (l *AddFriendByPhoneLogic) AddFriendByPhone(req *types.AddFriendByPhoneReq) (resp *types.BaseMsgResp, err error) {
	// todo: add your logic here and delete this line

	if len(req.Phones) == 0 {
		return nil, errors.New("miss phone")
	}

	orgID, ok := l.ctx.Value("organizationId").(uint64)
	if !ok || orgID == 0 {
		return nil, errors.New("content get oid info err")
	}

	//fmt.Println("before weChatIds:", req.WeChatIds)
	//fmt.Println("before phones:", req.Phones)
	//apiKeyObj.OrganizationID = 1

	if req.Type != 3 {
		req.Type = 1
	}
	infos, err := l.rebuildAddFriendByPhoneInfos(req, orgID)
	if err != nil {
		return nil, err
	}
	//fmt.Println("after weChatIds:", req.WeChatIds)
	//fmt.Println("after phones:", req.Phones)
	//fmt.Println(typekit.PrettyPrint(infos))

	var id int64
	for _, info := range infos {
		id, err = l.AppendWechaFriendAddInfo(&info)
		if err != nil {
			logx.Errorf("chat.AppendWechaFriendAddLog err : %s", err)
			return nil, err
		}
		logx.Infof("chat.AppendWechaFriendAddLog succ,get id:%d", id)

		continue
		if req.Type != 1 {
			continue //企微忽略后面
		}
		ret := addfriend.NewAddWechatFriendService(l.ctx, l.svcCtx).
			FindFriendByContent(info.WeChatId, info.Phone)
		retStr := "fail"
		if ret {
			retStr = "succ"
		}
		logx.Infof("call addfriend.NewAddWechatFriendService.FindFriendByContent('%s','%s') maybe %s", info.WeChatId, info.Phone, retStr)
	}
	return &types.BaseMsgResp{Msg: errormsg.Success, Code: errorcode.OK}, nil
}

func isDuplicateEntry(err error) bool {
	var mysqlErr *mysql.MySQLError
	if errors.As(err, &mysqlErr) {
		return mysqlErr.Number == 1062
	}
	return false
}

func (l *AddFriendByPhoneLogic) AppendWechaFriendAddInfo(addInfo *AddFriendByPhoneInfo) (int64, error) {

	isCanAdd := 0
	if addInfo.Type != 1 {
		isCanAdd = 1
	}
	isCanAdd = 1
	res, err := l.svcCtx.DB.AddWechatFriendLog.Create().
		SetNotNilOwnerWxID(&addInfo.WeChatId).
		SetNotNilOwnerWxType(&addInfo.Type).
		SetNotNilFindContent(&addInfo.Phone).
		SetNotNilMessage(&addInfo.Message).
		SetIsCanAdd(isCanAdd).
		SetNotNilOrganizationID(&addInfo.Oid).
		Save(l.ctx)

	id := int64(0)
	if err == nil {
		id = res.ID
	} else {
		if isDuplicateEntry(err) {
			err = errorx.NewDefaultError("手机号不能重复绑定")
		}
	}
	return id, err
}
