package MessageHandlers

import (
	"context"
	"time"

	"github.com/zeromicro/go-zero/core/logx"
	"wechat-api/internal/pkg/wechat_ws"
	"wechat-api/internal/svc"
)

type MessageHandlerStrategy interface {
	Handle(ctx context.Context, msg *wechat_ws.MsgJsonObject, svcCtx *svc.ServiceContext) error
}

type MessageHandler struct {
	strategies map[string]MessageHandlerStrategy
	svcCtx     *svc.ServiceContext
}

func NewMessageHandler(svcCtx *svc.ServiceContext) *MessageHandler {
	return &MessageHandler{
		strategies: make(map[string]MessageHandlerStrategy),
		svcCtx:     svcCtx,
	}
}

func (h *MessageHandler) GetWrappedHandler() func(msg *wechat_ws.MsgJsonObject) error {
	return WrapToSimpleHandler(h.Handle)
}

func (h *MessageHandler) RegisterStrategy(msgType string, strategy MessageHandlerStrategy) {
	h.strategies[msgType] = strategy
}

func (h *MessageHandler) RegisterStrategies(strategyMap map[string]func(*svc.ServiceContext) MessageHandlerStrategy) {
	for msgType, strategyFunc := range strategyMap {
		h.RegisterStrategy(msgType, strategyFunc(h.svcCtx))
		logx.Infof("已注册消息处理策略: %s", msgType)
	}
}

func (h *MessageHandler) Handle(ctx context.Context, msg *wechat_ws.MsgJsonObject) (err error) {
	defer func() {
		if r := recover(); r != nil {
			logx.Errorf("处理消息 msgType=%s 时 panic: %v", msg.MsgType, r)
			err = nil
		}
	}()

	strategy, exists := h.strategies[msg.MsgType]
	if !exists {
		logx.Infof("未注册处理器的消息类型: %s，跳过处理", msg.MsgType)
		return nil
	}
	return strategy.Handle(ctx, msg, h.svcCtx)
}

func toStrategy[T MessageHandlerStrategy](fn func(*svc.ServiceContext) T) func(*svc.ServiceContext) MessageHandlerStrategy {
	return func(svcCtx *svc.ServiceContext) MessageHandlerStrategy {
		return fn(svcCtx)
	}
}

func WrapToSimpleHandler(handler func(ctx context.Context, msg *wechat_ws.MsgJsonObject) error) func(msg *wechat_ws.MsgJsonObject) error {
	return func(msg *wechat_ws.MsgJsonObject) error {
		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
		defer cancel()
		return handler(ctx, msg)
	}
}
